
# q-expansion of newform 3888.2.a.f, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3888
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-2], [3], [2], [1], [8], [7], [-4], [-6], [1], [3], [6], [-1], [-6], [-2], [-10], [10], [-8], [-14], [10], [11], [4], [18], [17], [0], [8], [-6], [-11], [-6], [5], [22], [12], [-21], [20], [16], [7], [-9], [18], [-24], [-12], [-3], [-12], [14], [6], [17], [5], [-7], [0], [-23], [-8], [4], [21], [4], [14], [-16], [-16], [16], [13], [8], [-5], [-24], [12], [0], [-9], [24], [-11], [-3], [-6], [27], [-10], [-18], [-16], [11], [-11], [18], [4], [-37], [-18], [21], [6], [17], [28], [-17], [-36], [-8], [22], [-7], [16], [-11], [30], [-32], [-31], [30], [12], [24], [0], [-4], [-4], [-27], [28], [-14], [-4], [-2], [-39], [38], [20], [4], [-30], [-29], [-41], [-22], [12], [-4], [29], [34], [36], [-20], [18], [0], [-10], [-29], [46], [-16], [-23], [48], [17], [26], [0], [-19], [-35], [-4], [-15], [-34], [8], [11], [12], [-4], [-32], [2], [-37], [-6], [43], [-26], [25], [-30], [-22], [-54], [15], [-46], [3], [8], [13], [-56], [-13], [-16], [-16], [20], [27], [6], [-24], [0], [40], [-30], [6], [36], [-4], [10], [-34], [58], [24], [-54], [-50], [1], [-11], [14], [-53], [12], [-1], [11], [1], [-38], [-6], [42], [-36], [-42], [-2], [-9], [37], [-24], [-11], [66], [-25], [-14], [-42], [32], [-41], [-11], [-22], [30], [-16], [7], [-7], [-38], [6], [0], [-5], [42], [-22], [-31], [-33], [-60], [-64], [-68], [70], [14], [43], [22], [6], [48], [-31], [36], [-12], [-29], [48], [-1], [-36], [60], [39], [-58], [-51], [-19], [-47], [28], [-33], [-62], [29], [-44], [4], [-22], [12], [4], [56], [34], [54], [10], [47], [60], [-31], [-14], [54], [18], [4], [-14], [-54], [48], [62], [23], [12], [-29], [-7], [48], [-38], [-9], [-8], [61], [-34], [62], [1], [68], [-5], [43], [13], [32], [-35], [64], [-18], [-1], [-29], [-22], [-28], [-37], [-46], [-70], [-8], [8], [-19], [18], [4], [-34], [28], [-12], [-56], [78], [66], [16], [-23], [-60], [-60], [67], [-49], [26], [67], [38], [-69], [29], [-48], [14], [52], [35], [-12], [66], [-22], [-7], [-16], [-7], [0], [-18], [65], [-84], [-52], [-29], [14], [29], [-20], [-87], [-12], [84], [-10], [-6], [34], [-6], [-51], [66], [48], [90], [58], [-66], [-7], [-72], [-5], [6], [-66], [21], [-14], [-76], [-42], [-43], [42], [-34], [45], [-7], [-38], [0], [-22], [44], [-64], [-8], [58], [-30], [-37], [-12], [-18], [-86], [-43], [27], [-12], [-31], [67], [-56], [-69], [-68], [-20], [65], [5], [-12], [6], [-17], [36], [-22], [36], [-22], [9], [-24], [-13], [42], [-72], [77], [53], [18], [1], [34], [-12], [69], [-66], [58], [45], [0], [-53], [70], [-49], [-48], [23], [44], [70], [-13], [49], [-66], [-44], [-30], [23], [-26], [76], [44], [93], [-74], [-102], [67], [30], [-26], [50], [59], [94], [4], [-101], [62], [-4], [42], [48], [-16]]
